==> haystack-1.0.0-dev.0.3.tar.gz: test/HFilter_test.dart <==
///
/// Copyright (c) 2019, BAS Services & Graphics, LLC.
/// Licensed under the Academic Free License version 3.0
///

library haystack.filter_test;

import 'package:haystack/haystack.dart';
import 'package:haystack/src/HFilter.dart';
import 'package:test/test.dart';

import 'utils.dart';

void verifyParse(String s, HFilter expected) {
  HFilter actual = new HFilter.parse(s);
  expect(actual, expected);
}

class MockPather extends Pather {
  Map<String, HDict> map;
  MockPather( this.map );

  @override
  HDict find(String id) { return map[id]; }
}

void verifyInclude( Map<String, HDict> map, String query, String expected) {

  Pather db = new MockPather(map);

  HFilter q = new HFilter.parse(query);

  List<String> actual = [];
  map.keys.forEach ((id) {
    if (q.include(db.find(id), db)) {
      actual.add(id);
    }
  });

  expect(expected, actual.join(","));
}

void main() {

  group("HFilter", () {
    test("Identity", () {
      expect(HFilter.has("a"), HFilter.has("a"));
      expect(HFilter.has("a") != HFilter.has("b"), true);
    });

    test("Basics", () {
      verifyParse("x", HFilter.has("x"));
      verifyParse("foo", HFilter.has("foo"));
      verifyParse("fooBar", HFilter.has("fooBar"));
      verifyParse("foo7Bar", HFilter.has("foo7Bar"));
      verifyParse("foo_bar->a", HFilter.has("foo_bar->a"));
      verifyParse("a->b->c", HFilter.has("a->b->c"));
      verifyParse("not foo", HFilter.missing("foo"));
    });

    test("ZincOnlyLiteralsDontWork", () {
      expect(new HFilter.parse("x==T", false), isNull);
      expect(new HFilter.parse("x==F", false), isNull);
      expect(new HFilter.parse("x==F", false), isNull);
    });

    test("Bool", () {
      verifyParse("x->y==true", HFilter.eq("x->y", HBool.TRUE));
      verifyParse("x->y!=false", HFilter.ne("x->y", HBool.FALSE));
    });

    test("Str", () {
      verifyParse("x==\"hi\"", HFilter.eq("x", new HStr("hi")));
      verifyParse("x!=\"\\\"hi\\\"\"",  HFilter.ne("x", new HStr("\"hi\"")));
      verifyParse("x==\"_\\uabcd_\\n_\"", HFilter.eq("x", new HStr("_\uabcd_\n_")));
    });

    test("Uri", () {
      verifyParse("ref==`http://foo/?bar`", HFilter.eq("ref", new HUri("http://foo/?bar")));
      verifyParse("ref->x==`file name`", HFilter.eq("ref->x", new HUri("file name")));
      verifyParse("ref == `foo bar`", HFilter.eq("ref", new HUri("foo bar")));
    });

    test("Int", () {
      verifyParse("num < 4", HFilter.lt("num", n(4)));
      verifyParse("num <= -99", HFilter.le("num", n(-99)));
    });

    test("Float", () {
      verifyParse("num < 4.0", HFilter.lt("num", n(4.0)));
      verifyParse("num <= -9.6", HFilter.le("num", n(-9.6)));
      verifyParse("num > 400000", HFilter.gt("num", n(4e5)));
      verifyParse("num >= 16000", HFilter.ge("num", n(1.6e+4)));
      verifyParse("num >= 2.16", HFilter.ge("num", n(2.16)));
    });

    test("Unit", () {
      verifyParse("dur < 5ns", HFilter.lt("dur", n(5,"ns")));
      verifyParse("dur < 10kg", HFilter.lt("dur", n(10, "kg")));
      verifyParse("dur < -9sec", HFilter.lt("dur", n(-9, "sec")));
      verifyParse("dur < 2.5hr", HFilter.lt("dur", n(2.5, "hr")));
    });

    test("DateTime", () {
      verifyParse("foo < 2009-10-30", HFilter.lt("foo", new HDate.fromString("2009-10-30")));
      verifyParse("foo < 08:30:00", HFilter.lt("foo", new HTime.fromString("08:30:00")));
      verifyParse("foo < 13:00:00", HFilter.lt("foo", new HTime.fromString("13:00:00")));
    });

    test("Ref", () {
      verifyParse("author == @xyz", HFilter.eq("author", new HRef("xyz")));
      verifyParse("author==@xyz:foo.bar", HFilter.eq("author", new HRef("xyz:foo.bar")));
    });

    test("And", () {
      verifyParse("a and b", HFilter.has("a").and(HFilter.has("b")));
      verifyParse("a and b and c == 3", HFilter.has("a").and( HFilter.has("b").and(HFilter.eq("c", n(3))) ));
    });

    test("Or", () {
      verifyParse("a or b", HFilter.has("a").or(HFilter.has("b")));
      verifyParse("a or b or c == 3", HFilter.has("a").or(HFilter.has("b").or(HFilter.eq("c", n(3)))));
    });

    test("Parens", () {
      verifyParse("(a)", HFilter.has("a"));
      verifyParse("(a) and (b)", HFilter.has("a").and(HFilter.has("b")));
      verifyParse("( a )  and  ( b ) ", HFilter.has("a").and(HFilter.has("b")));
      verifyParse("(a or b) or (c == 3)", HFilter.has("a").or(HFilter.has("b")).or(HFilter.eq("c", n(3))));
    });

    test("Combo", () {
      HFilter isA = HFilter.has("a");
      HFilter isB = HFilter.has("b");
      HFilter isC = HFilter.has("c");
      HFilter isD = HFilter.has("d");
      verifyParse("a and b or c", (isA.and(isB)).or(isC));
      verifyParse("a or b and c", isA.or(isB.and(isC)));
      verifyParse("a and b or c and d", (isA.and(isB)).or(isC.and(isD)));
      verifyParse("(a and (b or c)) and d", isA.and(isB.or(isC)).and(isD));
      verifyParse("(a or (b and c)) or d", isA.or(isB.and(isC)).or(isD));
    });

    test("Include", () {
      HDict a = new HDictBuilder()
        .add("dis", "a")
        .add("num", 100)
        .add("foo", 99)
        .add("date", new HDate(2011,10,5))
        .toDict();

      HDict b = new HDictBuilder()
        .add("dis", "b")
        .add("num", 200)
        .add("foo", 88)
        .add("date", new HDate(2011,10,20))
        .add("bar")
        .add("ref", new HRef("a"))
        .toDict();

      HDict c = new HDictBuilder()
        .add("dis", "c")
        .add("num", 300)
        .add("ref", new HRef("b"))
        .add("bar")
        .toDict();

      var db = { "a": a, "b": b, "c": c };

      verifyInclude(db, "dis",                "a,b,c");
      verifyInclude(db, "dis == \"b\"",       "b");
      verifyInclude(db, "dis != \"b\"",       "a,c");
      verifyInclude(db, "dis <= \"b\"",       "a,b");
      verifyInclude(db, "dis >  \"b\"",       "c");
      verifyInclude(db, "num < 200",          "a");
      verifyInclude(db, "num <= 200",         "a,b");
      verifyInclude(db, "num > 200",          "c");
      verifyInclude(db, "num >= 200",         "b,c");
      verifyInclude(db, "date",               "a,b");
      verifyInclude(db, "date == 2011-10-20", "b");
      verifyInclude(db, "date < 2011-10-10",  "a");
      verifyInclude(db, "foo",                "a,b");
      verifyInclude(db, "not foo",            "c");
      verifyInclude(db, "foo == 88",          "b");
      verifyInclude(db, "foo != 88",          "a");
      verifyInclude(db, "foo == \"x\"",       "");
      verifyInclude(db, "ref",                "b,c");
      verifyInclude(db, "ref->dis",           "b,c");
      verifyInclude(db, "ref->dis == \"a\"",  "b");
      verifyInclude(db, "ref->bar",           "c");
      verifyInclude(db, "not ref->bar",       "a,b");
      verifyInclude(db, "foo and bar",        "b");
      verifyInclude(db, "foo or bar",         "a,b,c");
      verifyInclude(db, "(foo and bar) or num==300",  "b,c");
      verifyInclude(db, "foo and bar and num==300",   "");
    });


    test("Path", () {
      // single name
      Path path = new Path.parse("foo");
      expect(path.size, 1);
      expect(path.get(0), "foo");
      expect(path.toString(), "foo");
      expect(path, new Path.parse("foo"));

      // two names
      path = new Path.parse("foo->bar");
      expect(path.size, 2);
      expect(path.get(0), "foo");
      expect(path.get(1), "bar");
      expect(path.toString(), "foo->bar");
      expect(path, new Path.parse("foo->bar"));

      // three names
      path = new Path.parse("x->y->z");
      expect(path.size, 3);
      expect(path.get(0), "x");
      expect(path.get(1), "y");
      expect(path.get(2), "z");
      expect(path.toString(), "x->y->z");
      expect(path, new Path.parse("x->y->z"));
    });
  });

}

==> haystack-2.0.1.tar.gz: test/HFilter_test.dart <==
///
/// Copyright (c) 2019, BAS Services & Graphics, LLC.
/// Licensed under the Academic Free License version 3.0
///

library haystack.filter_test;

import 'package:haystack/haystack.dart';
import 'package:haystack/src/HFilter.dart';
import 'package:test/test.dart';

import 'utils.dart';

void verifyParse(String s, HFilter expected) {
  HFilter? actual = HFilter.parse(s);
  expect(actual, expected);
}

class MockPather extends Pather {
  Map<String, HDict> map;
  MockPather( this.map );

  @override
  HDict? find(String id) { return map[id]; }
}

void verifyInclude( Map<String, HDict> map, String query, String expected) {

  Pather db = MockPather(map);

  HFilter? q = HFilter.parse(query);

  expect(q, isNotNull);

  List<String> actual = [];
  map.keys.forEach ((id) {
    if (q!.include(db.find(id)!, db)) {
      actual.add(id);
    }
  });

  expect(expected, actual.join(','));
}

void main() {

  group("HFilter", () {
    test("Identity", () {
      expect(HFilter.has("a"), HFilter.has("a"));
      expect(HFilter.has("a") != HFilter.has("b"), true);
    });

    test("Basics", () {
      verifyParse("x", HFilter.has("x"));
      verifyParse("foo", HFilter.has("foo"));
      verifyParse("fooBar", HFilter.has("fooBar"));
      verifyParse("foo7Bar", HFilter.has("foo7Bar"));
      verifyParse("foo_bar->a", HFilter.has("foo_bar->a"));
      verifyParse("a->b->c", HFilter.has("a->b->c"));
      verifyParse("not foo", HFilter.missing("foo"));
    });

    test("ZincOnlyLiteralsDontWork", () {
      expect(HFilter.parse("x==T", false), isNull);
      expect(HFilter.parse("x==F", false), isNull);
      expect(HFilter.parse("x==F", false), isNull);
    });

    test("Bool", () {
      verifyParse("x->y==true", HFilter.eq("x->y", HBool.TRUE));
      verifyParse("x->y!=false", HFilter.ne("x->y", HBool.FALSE));
    });

    test("Str", () {
      verifyParse("x==\"hi\"", HFilter.eq("x", HStr("hi")));
      verifyParse("x!=\"\\\"hi\\\"\"",  HFilter.ne("x", HStr("\"hi\"")));
      verifyParse("x==\"_\\uabcd_\\n_\"", HFilter.eq("x", HStr("_\uabcd_\n_")));
    });

    test("Uri", () {
      verifyParse("ref==`http://foo/?bar`", HFilter.eq("ref", HUri("http://foo/?bar")));
      verifyParse("ref->x==`file name`", HFilter.eq("ref->x", HUri("file name")));
      verifyParse("ref == `foo bar`", HFilter.eq("ref", HUri("foo bar")));
    });

    test("Int", () {
      verifyParse("num < 4", HFilter.lt("num", n(4)));
      verifyParse("num <= -99", HFilter.le("num", n(-99)));
    });

    test("Float", () {
      verifyParse("num < 4.0", HFilter.lt("num", n(4.0)));
      verifyParse("num <= -9.6", HFilter.le("num", n(-9.6)));
      verifyParse("num > 400000", HFilter.gt("num", n(4e5)));
      verifyParse("num >= 16000", HFilter.ge("num", n(1.6e+4)));
      verifyParse("num >= 2.16", HFilter.ge("num", n(2.16)));
    });

    test("Unit", () {
      verifyParse("dur < 5ns", HFilter.lt("dur", n(5,"ns")));
      verifyParse("dur < 10kg", HFilter.lt("dur", n(10, "kg")));
      verifyParse("dur < -9sec", HFilter.lt("dur", n(-9, "sec")));
      verifyParse("dur < 2.5hr", HFilter.lt("dur", n(2.5, "hr")));
    });

    test("DateTime", () {
      verifyParse("foo < 2009-10-30", HFilter.lt("foo", HDate.fromString("2009-10-30")));
      verifyParse("foo < 08:30:00", HFilter.lt("foo", HTime.fromString("08:30:00")));
      verifyParse("foo < 13:00:00", HFilter.lt("foo", HTime.fromString("13:00:00")));
    });

    test("Ref", () {
      verifyParse("author == @xyz", HFilter.eq("author", HRef("xyz")));
      verifyParse("author==@xyz:foo.bar", HFilter.eq("author", HRef("xyz:foo.bar")));
    });

    test("And", () {
      verifyParse("a and b", HFilter.has("a").and(HFilter.has("b")));
      verifyParse("a and b and c == 3", HFilter.has("a").and( HFilter.has("b").and(HFilter.eq("c", n(3))) ));
    });

    test("Or", () {
      verifyParse("a or b", HFilter.has("a").or(HFilter.has("b")));
      verifyParse("a or b or c == 3", HFilter.has("a").or(HFilter.has("b").or(HFilter.eq("c", n(3)))));
    });

    test("Parens", () {
      verifyParse("(a)", HFilter.has("a"));
      verifyParse("(a) and (b)", HFilter.has("a").and(HFilter.has("b")));
      verifyParse("( a )  and  ( b ) ", HFilter.has("a").and(HFilter.has("b")));
      verifyParse("(a or b) or (c == 3)", HFilter.has("a").or(HFilter.has("b")).or(HFilter.eq("c", n(3))));
    });

    test("Combo", () {
      HFilter isA = HFilter.has("a");
      HFilter isB = HFilter.has("b");
      HFilter isC = HFilter.has("c");
      HFilter isD = HFilter.has("d");
      verifyParse("a and b or c", (isA.and(isB)).or(isC));
      verifyParse("a or b and c", isA.or(isB.and(isC)));
      verifyParse("a and b or c and d", (isA.and(isB)).or(isC.and(isD)));
      verifyParse("(a and (b or c)) and d", isA.and(isB.or(isC)).and(isD));
      verifyParse("(a or (b and c)) or d", isA.or(isB.and(isC)).or(isD));
    });

    test("Include", () {
      HDict a = HDictBuilder()
        .add("dis", "a")
        .add("num", 100)
        .add("foo", 99)
        .add("date", HDate(2011,10,5))
        .toDict();

      HDict b = HDictBuilder()
        .add("dis", "b")
        .add("num", 200)
        .add("foo", 88)
        .add("date", HDate(2011,10,20))
        .add("bar")
        .add("ref", HRef("a"))
        .toDict();

      HDict c = HDictBuilder()
        .add("dis", "c")
        .add("num", 300)
        .add("ref", HRef("b"))
        .add("bar")
        .toDict();

      var db = { "a": a, "b": b, "c": c };

      verifyInclude(db, "dis",                "a,b,c");
      verifyInclude(db, "dis == \"b\"",       "b");
      verifyInclude(db, "dis != \"b\"",       "a,c");
      verifyInclude(db, "dis <= \"b\"",       "a,b");
      verifyInclude(db, "dis >  \"b\"",       "c");
      verifyInclude(db, "num < 200",          "a");
      verifyInclude(db, "num <= 200",         "a,b");
      verifyInclude(db, "num > 200",          "c");
      verifyInclude(db, "num >= 200",         "b,c");
      verifyInclude(db, "date",               "a,b");
      verifyInclude(db, "date == 2011-10-20", "b");
      verifyInclude(db, "date < 2011-10-10",  "a");
      verifyInclude(db, "foo",                "a,b");
      verifyInclude(db, "not foo",            "c");
      verifyInclude(db, "foo == 88",          "b");
      verifyInclude(db, "foo != 88",          "a");
      verifyInclude(db, "foo == \"x\"",       "");
      verifyInclude(db, "ref",                "b,c");
      verifyInclude(db, "ref->dis",           "b,c");
      verifyInclude(db, "ref->dis == \"a\"",  "b");
      verifyInclude(db, "ref->bar",           "c");
      verifyInclude(db, "not ref->bar",       "a,b");
      verifyInclude(db, "foo and bar",        "b");
      verifyInclude(db, "foo or bar",         "a,b,c");
      verifyInclude(db, "(foo and bar) or num==300",  "b,c");
      verifyInclude(db, "foo and bar and num==300",   "");
    });


    test("Path", () {
      // single name
      Path path = Path.parse("foo");
      expect(path.size, 1);
      expect(path.get(0), "foo");
      expect(path.toString(), "foo");
      expect(path, Path.parse("foo"));

      // two names
      path = Path.parse("foo->bar");
      expect(path.size, 2);
      expect(path.get(0), "foo");
      expect(path.get(1), "bar");
      expect(path.toString(), "foo->bar");
      expect(path, Path.parse("foo->bar"));

      // three names
      path = Path.parse("x->y->z");
      expect(path.size, 3);
      expect(path.get(0), "x");
      expect(path.get(1), "y");
      expect(path.get(2), "z");
      expect(path.toString(), "x->y->z");
      expect(path, Path.parse("x->y->z"));
    });
  });

}
